import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  QueryList,
  ViewChild,
  ViewChildren,
  ViewEncapsulation,
} from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { MatCheckbox, MatCheckboxChange } from '@angular/material/checkbox';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import {
  debounceTime,
  map,
  merge,
  Observable,
  of,
  Subject,
  switchMap,
  takeUntil,
} from 'rxjs';
import { fuseAnimations } from '@fuse/animations';
import { FuseConfirmationService } from '@fuse/services/confirmation';

import { Deposit } from '../models/deposit';
import { DepositPagination } from '../models/deposit-pagination';
import { DepositService } from '../services/deposit.service';
import { ActivatedRoute, Router } from '@angular/router';
import {
  ListMemberBankDepositsResponse,
  UpdateMemberBankDepositForStateRequest,
} from 'app/modules/proto/c2se/member_bank_deposit_pb';
import {
  MemberBankDeposit,
  MemberBankDepositModel,
  MemberBankDepositState,
} from 'app/modules/proto/models/member_bank_deposit_pb';
import { MemberModel } from 'app/modules/proto/models/member_pb';
import { ListMembersResponse } from 'app/modules/proto/c2se/member_pb';
import { ListMemberClassesResponse } from 'app/modules/proto/c2se/member_class_pb';
import { MemberClass } from 'app/modules/proto/models/member_class_pb';
import { MemberBankDepositService } from 'app/modules/polyglot/member_bank_deposit/services/member_bank_deposit.service';

@Component({
  selector: 'deposit-list',
  templateUrl: './list.component.html',
  styles: [
    /* language=SCSS */
    `
      .deposit-grid {
        /*                     CB   상부  아이디  등급  예금주 정산종류 */
        grid-template-columns: 20px 80px 100px auto 60px 140px;

        @screen sm {
          /*                     CB   상부  아이디  등급  예금주 정산종류 비고 예정  등록*/
          grid-template-columns: 20px 80px 100px auto 60px 140px 120px;
        }

        @screen md {
          /*                     CB   상부  아이디  등급  예금주 정산종류 비고 예정 등록  입금출금 */
          grid-template-columns: 20px 80px 100px auto 60px 140px 120px 140px;
        }

        @screen lg {
          /*                     CB   구분   아이디  입금정보  정산종류  등록날짜 입금출금 게임중머니 카지노  상위  배팅 삭제*/
          grid-template-columns: 20px 80px 100px auto 60px 140px 120px 140px 80px 120px 80px 60px;
        }
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class ListComponent implements OnInit, AfterViewInit, OnDestroy {
  @ViewChild(MatPaginator) private _paginator!: MatPaginator;
  @ViewChild(MatSort) private _sort!: MatSort;

  @ViewChildren('chkDeposits') chkDeposits!: QueryList<MatCheckbox>;

  listMember!: MemberModel[];
  memberClasses!: MemberClass[];
  bankDeposit$!: Observable<MemberBankDepositModel[] | undefined>;
  __checkedDeposits: string[] = [];

  __isSearchOpened = false;
  isLoading = false;
  searchInputControl = new FormControl();
  selectedDeposit?: Deposit;
  pagination?: DepositPagination;
  MemberBankDepositState = MemberBankDepositState;

  private _unsubscribeAll: Subject<any> = new Subject<any>();

  /**
   * Constructor
   */
  constructor(
    private _activatedRoute: ActivatedRoute,
    private _changeDetectorRef: ChangeDetectorRef,
    private _fuseConfirmationService: FuseConfirmationService,
    private _formBuilder: FormBuilder,
    private _depositService: DepositService,
    private router: Router,
    private _bankDepositService: MemberBankDepositService
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    // Get the pagination
    this._depositService.pagination$
      .pipe(takeUntil(this._unsubscribeAll))
      .subscribe((pagination: DepositPagination | undefined) => {
        // Update the pagination
        this.pagination = pagination;

        // Mark for check
        this._changeDetectorRef.markForCheck();
      });

    // Get the products

    this._activatedRoute.data.subscribe((data) => {
      let listMemberResult: ListMembersResponse.Result = data['listMember'];
      let listBankDepositResult: ListMemberBankDepositsResponse.Result =
        data['bankDeposit'];
      let listMemberClasses: ListMemberClassesResponse.Result =
        data['listMemberClasses'];

      this.listMember = listMemberResult.getMembersList();
      this.bankDeposit$ = of(listBankDepositResult.getMemberBankDepositsList());
      this.memberClasses = listMemberClasses.getMemberClassesList();

      console.log(
        'bankDeposit: ',
        listBankDepositResult.getMemberBankDepositsList()
      );

      // Mark for check
      this._changeDetectorRef.markForCheck();
    });
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {
    if (this._sort && this._paginator) {
      // Set the initial sort
      this._sort.sort({
        id: 'signinId',
        start: 'asc',
        disableClear: true,
      });

      // Mark for check
      this._changeDetectorRef.markForCheck();

      // If the deposit changes the sort order...
      this._sort.sortChange
        .pipe(takeUntil(this._unsubscribeAll))
        .subscribe(() => {
          // Reset back to the first page
          this._paginator.pageIndex = 0;
        });

      // Get products if sort or page changes
      merge(this._sort.sortChange, this._paginator.page)
        .pipe(
          switchMap(() => {
            this.isLoading = true;
            return this._depositService.getDeposits(
              this._paginator.pageIndex,
              this._paginator.pageSize,
              this._sort.active,
              this._sort.direction
            );
          }),
          map(() => {
            this.isLoading = false;
          })
        )
        .subscribe();
    }
  }

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  viewUserDetail(id: string | undefined): void {
    if (!id) {
      return;
    }
    let url: string = 'member/user/' + id;
    this.router.navigateByUrl(url);
  }
  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * toggle the search
   * Used in 'bar'
   */
  __onClickSearch(): void {
    this.__isSearchOpened = !this.__isSearchOpened;
  }

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }

  onClickDeposit(typeNum: number): void {
    if (this.__checkedDeposits.length === 0) {
      return;
    }

    let state =
      typeNum === 3
        ? MemberBankDepositState.COMPLETE
        : MemberBankDepositState.PENDING;
    let req: UpdateMemberBankDepositForStateRequest;

    this.__checkedDeposits.map((v) => {
      req = new UpdateMemberBankDepositForStateRequest();
      req.setId(v);
      req.setState(state);

      this._bankDepositService
        .updateMemberBankDepositForState(req)
        .then((result) => {
          console.log(result);
        })
        .catch((reson) => console.log(reson));
    });
  }

  __onChangeChkDepositsAll(event: MatCheckboxChange): void {
    let checked = event.checked;
    this.chkDeposits.forEach((c, i) => {
      c.checked = checked;
    });
    this.__updateCheckedDeposits();
  }
  __onChangeChkDeposits(event: MatCheckboxChange): void {
    this.__updateCheckedDeposits();
  }

  __updateCheckedDeposits(): void {
    this.__checkedDeposits = [];
    this.chkDeposits.forEach((c, i) => {
      if (c.checked) {
        this.__checkedDeposits.push(c.value);
      }
    });
  }

  __getMemberRefferByMemberId(
    memberId: string | undefined
  ): string | undefined {
    if (!memberId) {
      return;
    }
    const member = this.__getMemberById(memberId);

    if (!!member) {
      const parentUserName = member.getParentMember()?.getUsername();
      const parentClassesId = member
        .getParentMember()
        ?.getMemberClass()
        ?.getId();
      const parentClass = this.memberClasses.find(
        (v) => v.getId() === parentClassesId
      );

      const parentClassName = parentClass?.getName();

      return `[${parentClassName}]${parentUserName}`;
    }

    return undefined;
  }
  __getDepositState(deposit: MemberBankDepositModel): string | undefined {
    const state = deposit.getState();
    let result: string = '';

    switch (state) {
      case MemberBankDepositState.APPLICATION:
        result = '신청';
        break;
      case MemberBankDepositState.PENDING:
        result = '대기';
        break;
      case MemberBankDepositState.COMPLETE:
        result = '완료';
        break;
      default:
        break;
    }

    return result;
  }

  __getMemberSettlementByMemberId(
    memberId: string | undefined
  ): string | undefined {
    if (!memberId) {
      return;
    }

    const member = this.__getMemberById(memberId);

    if (!!member) {
      const type = member.getMemberSettlementSetting()?.getSettlementType();
      return type === 1 ? '롤링' : '';
    }

    return undefined;
  }
  __getMemberLevelByMemberId(memberId: string | undefined): string | undefined {
    if (!memberId) {
      return;
    }
    const member = this.__getMemberById(memberId);

    if (!!member) {
      return member.getMemberLevel()?.getName();
    }

    return undefined;
  }

  __getMemberBankByMemberId(memberId: string | undefined): string | undefined {
    if (!memberId) {
      return;
    }
    const member = this.__getMemberById(memberId);

    if (!!member) {
      return member.getBankAccount()?.getBank()?.getName();
    }

    return undefined;
  }

  __getMemberClassesByMemberId(
    memberId: string | undefined
  ): string | undefined {
    if (!memberId) {
      return;
    }
    const member = this.__getMemberById(memberId);

    if (!!member) {
      return member.getMemberClass()?.getName();
    }

    return undefined;
  }
  __getMemberById(memberId: string | undefined): MemberModel | undefined {
    if (!memberId) {
      return;
    }
    return this.listMember.find((v) => v.getId() === memberId);
  }
}
